# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.goa_attribute_tiny_request_body import GoaAttributeTinyRequestBody
from ..types.trailers_create_trailer_response_body import TrailersCreateTrailerResponseBody
from ..types.trailers_get_trailer_response_body import TrailersGetTrailerResponseBody
from ..types.trailers_list_trailers_response_body import TrailersListTrailersResponseBody
from ..types.trailers_update_trailer_response_body import TrailersUpdateTrailerResponseBody
from .raw_client import AsyncRawTrailersClient, RawTrailersClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TrailersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTrailersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTrailersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTrailersClient
        """
        return self._raw_client

    def list_trailers(
        self,
        *,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailersListTrailersResponseBody:
        """
        List all trailers.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailersListTrailersResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.trailers.list_trailers()
        """
        _response = self._raw_client.list_trailers(
            tag_ids=tag_ids, parent_tag_ids=parent_tag_ids, limit=limit, after=after, request_options=request_options
        )
        return _response.data

    def create_trailer(
        self,
        *,
        name: str,
        attributes: typing.Optional[typing.Sequence[GoaAttributeTinyRequestBody]] = OMIT,
        enabled_for_mobile: typing.Optional[bool] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        license_plate: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        trailer_serial_number: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailersCreateTrailerResponseBody:
        """
        Creates a new trailer asset.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        name : str
            The human-readable name of the Trailer. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. By default, this name is the serial number of the Samsara Asset Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.

        attributes : typing.Optional[typing.Sequence[GoaAttributeTinyRequestBody]]
            A list of attributes to assign to the trailer.

        enabled_for_mobile : typing.Optional[bool]
            Indicates if the trailer is visible on the Samsara mobile apps.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        license_plate : typing.Optional[str]
            The license plate of the Trailer. **By default**: empty. Can be set or updated through the Samsara Dashboard or the API at any time.

        notes : typing.Optional[str]
            These are generic notes about the Trailer. Empty by default. Can be set or updated through the Samsara Dashboard or the API at any time.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this trailer. If your access to the API is scoped by one or more tags, this field is required to pass in.

        trailer_serial_number : typing.Optional[str]
            The serial number of the trailer.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailersCreateTrailerResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.trailers.create_trailer(
            name="Trailer-123",
        )
        """
        _response = self._raw_client.create_trailer(
            name=name,
            attributes=attributes,
            enabled_for_mobile=enabled_for_mobile,
            external_ids=external_ids,
            license_plate=license_plate,
            notes=notes,
            tag_ids=tag_ids,
            trailer_serial_number=trailer_serial_number,
            request_options=request_options,
        )
        return _response.data

    def get_trailer(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TrailersGetTrailerResponseBody:
        """
        Retrieve a trailer with given ID.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the trailer. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: "key:value". For example, "maintenanceId:250020".

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailersGetTrailerResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.trailers.get_trailer(
            id="id",
        )
        """
        _response = self._raw_client.get_trailer(id, request_options=request_options)
        return _response.data

    def delete_trailer(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a trailer with the given ID.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the trailer to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.trailers.delete_trailer(
            id="id",
        )
        """
        _response = self._raw_client.delete_trailer(id, request_options=request_options)
        return _response.data

    def update_trailer(
        self,
        id: str,
        *,
        attributes: typing.Optional[typing.Sequence[GoaAttributeTinyRequestBody]] = OMIT,
        enabled_for_mobile: typing.Optional[bool] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        license_plate: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        odometer_meters: typing.Optional[int] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        trailer_serial_number: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailersUpdateTrailerResponseBody:
        """
        Update a trailer.  **Note** this implementation of patch uses [the JSON merge patch](https://tools.ietf.org/html/rfc7396) proposed standard.
         This means that any fields included in the patch request will _overwrite_ fields which exist on the target resource.
         For arrays, this means any array included in the request will _replace_ the array that exists at the specified path, it will not _add_ to the existing array

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the trailer. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the trailer.

        attributes : typing.Optional[typing.Sequence[GoaAttributeTinyRequestBody]]
            A list of attributes to assign to the trailer.

        enabled_for_mobile : typing.Optional[bool]
            Indicates if the trailer is visible on the Samsara mobile apps.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        license_plate : typing.Optional[str]
            The license plate of the Trailer. **By default**: empty. Can be set or updated through the Samsara Dashboard or the API at any time.

        name : typing.Optional[str]
            The human-readable name of the Trailer. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. By default, this name is the serial number of the Samsara Asset Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.

        notes : typing.Optional[str]
            These are generic notes about the Trailer. Empty by default. Can be set or updated through the Samsara Dashboard or the API at any time.

        odometer_meters : typing.Optional[int]
            When you provide a manual odometer override, Samsara will begin updating a trailer's odometer using GPS distance traveled since this override was set. Only applies to trailers installed with Samsara gateways.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this trailer. If your access to the API is scoped by one or more tags, this field is required to pass in.

        trailer_serial_number : typing.Optional[str]
            The serial number of the trailer.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailersUpdateTrailerResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.trailers.update_trailer(
            id="id",
        )
        """
        _response = self._raw_client.update_trailer(
            id,
            attributes=attributes,
            enabled_for_mobile=enabled_for_mobile,
            external_ids=external_ids,
            license_plate=license_plate,
            name=name,
            notes=notes,
            odometer_meters=odometer_meters,
            tag_ids=tag_ids,
            trailer_serial_number=trailer_serial_number,
            request_options=request_options,
        )
        return _response.data


class AsyncTrailersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTrailersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTrailersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTrailersClient
        """
        return self._raw_client

    async def list_trailers(
        self,
        *,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailersListTrailersResponseBody:
        """
        List all trailers.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailersListTrailersResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.trailers.list_trailers()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_trailers(
            tag_ids=tag_ids, parent_tag_ids=parent_tag_ids, limit=limit, after=after, request_options=request_options
        )
        return _response.data

    async def create_trailer(
        self,
        *,
        name: str,
        attributes: typing.Optional[typing.Sequence[GoaAttributeTinyRequestBody]] = OMIT,
        enabled_for_mobile: typing.Optional[bool] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        license_plate: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        trailer_serial_number: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailersCreateTrailerResponseBody:
        """
        Creates a new trailer asset.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        name : str
            The human-readable name of the Trailer. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. By default, this name is the serial number of the Samsara Asset Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.

        attributes : typing.Optional[typing.Sequence[GoaAttributeTinyRequestBody]]
            A list of attributes to assign to the trailer.

        enabled_for_mobile : typing.Optional[bool]
            Indicates if the trailer is visible on the Samsara mobile apps.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        license_plate : typing.Optional[str]
            The license plate of the Trailer. **By default**: empty. Can be set or updated through the Samsara Dashboard or the API at any time.

        notes : typing.Optional[str]
            These are generic notes about the Trailer. Empty by default. Can be set or updated through the Samsara Dashboard or the API at any time.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this trailer. If your access to the API is scoped by one or more tags, this field is required to pass in.

        trailer_serial_number : typing.Optional[str]
            The serial number of the trailer.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailersCreateTrailerResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.trailers.create_trailer(
                name="Trailer-123",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_trailer(
            name=name,
            attributes=attributes,
            enabled_for_mobile=enabled_for_mobile,
            external_ids=external_ids,
            license_plate=license_plate,
            notes=notes,
            tag_ids=tag_ids,
            trailer_serial_number=trailer_serial_number,
            request_options=request_options,
        )
        return _response.data

    async def get_trailer(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TrailersGetTrailerResponseBody:
        """
        Retrieve a trailer with given ID.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the trailer. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: "key:value". For example, "maintenanceId:250020".

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailersGetTrailerResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.trailers.get_trailer(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_trailer(id, request_options=request_options)
        return _response.data

    async def delete_trailer(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a trailer with the given ID.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Unique identifier for the trailer to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.trailers.delete_trailer(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_trailer(id, request_options=request_options)
        return _response.data

    async def update_trailer(
        self,
        id: str,
        *,
        attributes: typing.Optional[typing.Sequence[GoaAttributeTinyRequestBody]] = OMIT,
        enabled_for_mobile: typing.Optional[bool] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        license_plate: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        odometer_meters: typing.Optional[int] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        trailer_serial_number: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailersUpdateTrailerResponseBody:
        """
        Update a trailer.  **Note** this implementation of patch uses [the JSON merge patch](https://tools.ietf.org/html/rfc7396) proposed standard.
         This means that any fields included in the patch request will _overwrite_ fields which exist on the target resource.
         For arrays, this means any array included in the request will _replace_ the array that exists at the specified path, it will not _add_ to the existing array

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Trailers** under the Trailers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the trailer. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the trailer.

        attributes : typing.Optional[typing.Sequence[GoaAttributeTinyRequestBody]]
            A list of attributes to assign to the trailer.

        enabled_for_mobile : typing.Optional[bool]
            Indicates if the trailer is visible on the Samsara mobile apps.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        license_plate : typing.Optional[str]
            The license plate of the Trailer. **By default**: empty. Can be set or updated through the Samsara Dashboard or the API at any time.

        name : typing.Optional[str]
            The human-readable name of the Trailer. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. By default, this name is the serial number of the Samsara Asset Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.

        notes : typing.Optional[str]
            These are generic notes about the Trailer. Empty by default. Can be set or updated through the Samsara Dashboard or the API at any time.

        odometer_meters : typing.Optional[int]
            When you provide a manual odometer override, Samsara will begin updating a trailer's odometer using GPS distance traveled since this override was set. Only applies to trailers installed with Samsara gateways.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this trailer. If your access to the API is scoped by one or more tags, this field is required to pass in.

        trailer_serial_number : typing.Optional[str]
            The serial number of the trailer.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailersUpdateTrailerResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.trailers.update_trailer(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_trailer(
            id,
            attributes=attributes,
            enabled_for_mobile=enabled_for_mobile,
            external_ids=external_ids,
            license_plate=license_plate,
            name=name,
            notes=notes,
            odometer_meters=odometer_meters,
            tag_ids=tag_ids,
            trailer_serial_number=trailer_serial_number,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .patch_job_objectjob_location_request_object_request_body import PatchJobObjectjobLocationRequestObjectRequestBody


class PatchJobObjectRequestBody(UniversalBaseModel):
    """
    Job object with fields to update. If a field is not provided, it will not be updated
    """

    address: typing.Optional[PatchJobObjectjobLocationRequestObjectRequestBody] = None
    customer_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="customerName"),
        pydantic.Field(alias="customerName", description="Customer name for job"),
    ] = None
    end_date: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endDate"),
        pydantic.Field(
            alias="endDate",
            description="End date of job in RFC 3339 format. Must be greater than or equal to the start date",
        ),
    ] = None
    fleet_device_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[int]],
        FieldMetadata(alias="fleetDeviceIds"),
        pydantic.Field(
            alias="fleetDeviceIds",
            description="Fleet devices to be added to this job (cannot have both industrial assets and fleet devices in the same job)",
        ),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Job Id
    """

    industrial_asset_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="industrialAssetIds"),
        pydantic.Field(
            alias="industrialAssetIds",
            description="IndustrialAssets to be added to this job (cannot have both industrial assets and fleet devices in the same job)",
        ),
    ] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Job name
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes for the upcoming job
    """

    ontime_window_after_arrival_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ontimeWindowAfterArrivalMs"),
        pydantic.Field(
            alias="ontimeWindowAfterArrivalMs",
            description="Specifies the time window (in milliseconds) after a stop's scheduled arrival time during which the stop is considered 'on-time'.",
        ),
    ] = None
    ontime_window_before_arrival_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ontimeWindowBeforeArrivalMs"),
        pydantic.Field(
            alias="ontimeWindowBeforeArrivalMs",
            description="Specifies the time window (in milliseconds) before a stop's scheduled arrival time during which the stop is considered 'on-time'.",
        ),
    ] = None
    start_date: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="startDate"),
        pydantic.Field(alias="startDate", description="Start date of job in RFC 3339 format."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..types.inline_response_2001 import InlineResponse2001


class RawLegacyClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def v_1_get_all_assets(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[InlineResponse2001]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/listassets) instead. The endpoint will continue to function as documented.**

         Fetch all powered and unpowered equipment.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InlineResponse2001]
            List of assets.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/fleet/assets",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InlineResponse2001,
                    parse_obj_as(
                        type_=InlineResponse2001,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLegacyClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def v_1_get_all_assets(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[InlineResponse2001]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/listassets) instead. The endpoint will continue to function as documented.**

         Fetch all powered and unpowered equipment.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InlineResponse2001]
            List of assets.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/fleet/assets",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InlineResponse2001,
                    parse_obj_as(
                        type_=InlineResponse2001,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

HealthResponseResponseBodyHealthStatus = typing.Union[
    typing.Literal["dataPending", "healthy", "needsAttention", "needsReplacement"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_geofence_object_response_body_entry_type import FormsGeofenceObjectResponseBodyEntryType


class FormsGeofenceObjectResponseBody(UniversalBaseModel):
    """
    Tracked or untracked (i.e. manually entered) geofence object.
    """

    address: typing.Optional[str] = pydantic.Field(default=None)
    """
    Address of the geofence. Included if 'entryType' is `tracked`.
    """

    entry_type: typing_extensions.Annotated[
        FormsGeofenceObjectResponseBodyEntryType,
        FieldMetadata(alias="entryType"),
        pydantic.Field(
            alias="entryType", description="The type of entry for the geofence.  Valid values: `tracked`, `untracked`"
        ),
    ]
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the tracked geofence. Included if 'entryType' is `tracked`.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of an untracked (i.e. manually entered) geofence.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .safety_score_details_request_body_comparison import SafetyScoreDetailsRequestBodyComparison


class SafetyScoreDetailsRequestBody(UniversalBaseModel):
    """
    Trigger when safety score meets the specified condition.
    """

    comparison: SafetyScoreDetailsRequestBodyComparison = pydantic.Field()
    """
    The comparison to use when comparing the value to the threshold.  Valid values: `EQUAL_TO`, `GREATER_THAN`, `GREATER_THAN_OR_EQUAL_TO`, `LESS_THAN`, `LESS_THAN_OR_EQUAL_TO`
    """

    score: int = pydantic.Field()
    """
    The score to compare to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .author_signature_object_response_body_type import AuthorSignatureObjectResponseBodyType
from .signatory_user_object_response_body import SignatoryUserObjectResponseBody


class AuthorSignatureObjectResponseBody(UniversalBaseModel):
    """
    An author signature for DVIRs with a signed time.
    """

    signatory_user: typing_extensions.Annotated[
        SignatoryUserObjectResponseBody, FieldMetadata(alias="signatoryUser"), pydantic.Field(alias="signatoryUser")
    ]
    signed_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="signedAtTime"),
        pydantic.Field(
            alias="signedAtTime", description="The time when the DVIR was signed. UTC timestamp in RFC 3339 format."
        ),
    ]
    type: AuthorSignatureObjectResponseBodyType = pydantic.Field()
    """
    Whether the DVIR was submitted by a driver or mechanic.  Valid values: `driver`, `mechanic`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CarbCtcVehicleHistoryObjectResponseBodyTestResult = typing.Union[
    typing.Literal["pass", "fail", "error", "unknown"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_tiny_response import DriverTinyResponse
from .hos_log_entry_hos_status_type import HosLogEntryHosStatusType
from .hos_log_location import HosLogLocation
from .time import Time
from .vehicle_tiny_response import VehicleTinyResponse


class HosLogEntry(UniversalBaseModel):
    """
    A single HOS log entry.
    """

    codrivers: typing.Optional[typing.List[DriverTinyResponse]] = pydantic.Field(default=None)
    """
    The codriver information.
    """

    hos_status_type: typing_extensions.Annotated[
        typing.Optional[HosLogEntryHosStatusType],
        FieldMetadata(alias="hosStatusType"),
        pydantic.Field(
            alias="hosStatusType",
            description="The Hours of Service status type. Valid values: `offDuty`, `sleeperBed`, `driving`, `onDuty`, `yardMove`, `personalConveyance`.",
        ),
    ] = None
    log_end_time: typing_extensions.Annotated[
        typing.Optional[Time], FieldMetadata(alias="logEndTime"), pydantic.Field(alias="logEndTime")
    ] = None
    log_recorded_location: typing_extensions.Annotated[
        typing.Optional[HosLogLocation],
        FieldMetadata(alias="logRecordedLocation"),
        pydantic.Field(alias="logRecordedLocation"),
    ] = None
    log_start_time: typing_extensions.Annotated[
        Time, FieldMetadata(alias="logStartTime"), pydantic.Field(alias="logStartTime")
    ]
    remark: typing.Optional[str] = pydantic.Field(default=None)
    """
    Remark associated with the log entry.
    """

    vehicle: typing.Optional[VehicleTinyResponse] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .safety_score_behavior_object_response_body import SafetyScoreBehaviorObjectResponseBody
from .safety_score_speeding_object_response_body import SafetyScoreSpeedingObjectResponseBody


class TripsBreakdownResponseBody(UniversalBaseModel):
    """
    Trip impacts on safety score.
    """

    behaviors: typing.List[SafetyScoreBehaviorObjectResponseBody] = pydantic.Field()
    """
    Aggregated list of behaviors that occurred during the trip, one row for each unique type of behavior.
    """

    drive_distance_meters: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="driveDistanceMeters"),
        pydantic.Field(alias="driveDistanceMeters", description="Distance driven during the trip."),
    ]
    driver_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="driverId"),
        pydantic.Field(alias="driverId", description="ID of the driver associated with the trip."),
    ]
    speeding: typing.List[SafetyScoreSpeedingObjectResponseBody] = pydantic.Field()
    """
    Aggregated list of speeding events that occurred during the trip, one row for each unique type of speeding.
    """

    trip_end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="tripEndTime"),
        pydantic.Field(alias="tripEndTime", description="[RFC 3339] Time the trip ended in UTC."),
    ]
    trip_score_impact: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="tripScoreImpact"),
        pydantic.Field(
            alias="tripScoreImpact",
            description="Impact of the trip to the overall score. This is the effective sum of all risk factors in the trip.",
        ),
    ]
    trip_start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="tripStartTime"),
        pydantic.Field(alias="tripStartTime", description="[RFC 3339] Time the trip started in UTC."),
    ]
    vehicle_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="vehicleId"),
        pydantic.Field(alias="vehicleId", description="ID of the vehicle associated with the trip."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DeviceRecoveryRecoverAssetRequestBodyMissingReason = typing.Union[
    typing.Literal["MISPLACED", "STOLEN", "NOT_SURE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

WorkOrdersPatchWorkOrdersRequestBodyPriority = typing.Union[
    typing.Literal["High", "Low", "Medium", "Urgent"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .trailer_stat_gps_odometer_meters_with_decorations_type_response_body import (
    TrailerStatGpsOdometerMetersWithDecorationsTypeResponseBody,
)
from .trailer_stat_gps_with_decorations_type_response_body import TrailerStatGpsWithDecorationsTypeResponseBody
from .trailer_stat_reefer_alarm_with_decorations_type_response_body import (
    TrailerStatReeferAlarmWithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_ambient_air_temperature_with_decorations_type_response_body import (
    TrailerStatReeferAmbientAirTemperatureWithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_door_state_zone_1_with_decorations_type_response_body import (
    TrailerStatReeferDoorStateZone1WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_door_state_zone_2_with_decorations_type_response_body import (
    TrailerStatReeferDoorStateZone2WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_door_state_zone_3_with_decorations_type_response_body import (
    TrailerStatReeferDoorStateZone3WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_fuel_percent_with_decorations_type_response_body import (
    TrailerStatReeferFuelPercentWithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_obd_engine_seconds_with_decorations_type_response_body import (
    TrailerStatReeferObdEngineSecondsWithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_return_air_temperature_milli_c_zone_1_with_decorations_type_response_body import (
    TrailerStatReeferReturnAirTemperatureMilliCZone1WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_return_air_temperature_milli_c_zone_2_with_decorations_type_response_body import (
    TrailerStatReeferReturnAirTemperatureMilliCZone2WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_return_air_temperature_milli_c_zone_3_with_decorations_type_response_body import (
    TrailerStatReeferReturnAirTemperatureMilliCZone3WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_run_mode_with_decorations_type_response_body import (
    TrailerStatReeferRunModeWithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_set_point_temperature_milli_c_zone_1_with_decorations_type_response_body import (
    TrailerStatReeferSetPointTemperatureMilliCZone1WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_set_point_temperature_milli_c_zone_2_with_decorations_type_response_body import (
    TrailerStatReeferSetPointTemperatureMilliCZone2WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_set_point_temperature_milli_c_zone_3_with_decorations_type_response_body import (
    TrailerStatReeferSetPointTemperatureMilliCZone3WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_state_with_decorations_type_response_body import (
    TrailerStatReeferStateWithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_state_zone_1_with_decorations_type_response_body import (
    TrailerStatReeferStateZone1WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_state_zone_2_with_decorations_type_response_body import (
    TrailerStatReeferStateZone2WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_state_zone_3_with_decorations_type_response_body import (
    TrailerStatReeferStateZone3WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_supply_air_temperature_milli_c_zone_1_with_decorations_type_response_body import (
    TrailerStatReeferSupplyAirTemperatureMilliCZone1WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_supply_air_temperature_milli_c_zone_2_with_decorations_type_response_body import (
    TrailerStatReeferSupplyAirTemperatureMilliCZone2WithDecorationsTypeResponseBody,
)
from .trailer_stat_reefer_supply_air_temperature_milli_c_zone_3_with_decorations_type_response_body import (
    TrailerStatReeferSupplyAirTemperatureMilliCZone3WithDecorationsTypeResponseBody,
)


class TrailerStatsObjectResponseBody(UniversalBaseModel):
    """
    A trailer and its stats
    """

    carrier_reefer_state: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferStateWithDecorationsTypeResponseBody]],
        FieldMetadata(alias="carrierReeferState"),
        pydantic.Field(alias="carrierReeferState", description="A list of engine state points."),
    ] = None
    gps: typing.Optional[typing.List[TrailerStatGpsWithDecorationsTypeResponseBody]] = pydantic.Field(default=None)
    """
    A list of GPS points.
    """

    gps_odometer_meters: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatGpsOdometerMetersWithDecorationsTypeResponseBody]],
        FieldMetadata(alias="gpsOdometerMeters"),
        pydantic.Field(alias="gpsOdometerMeters", description="A list of odometer points."),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the trailer.
    """

    name: str = pydantic.Field()
    """
    Name of the vehicle.
    """

    reefer_alarms: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferAlarmWithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferAlarms"),
        pydantic.Field(alias="reeferAlarms", description="A list of reefer alarm points."),
    ] = None
    reefer_ambient_air_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferAmbientAirTemperatureWithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferAmbientAirTemperatureMilliC"),
        pydantic.Field(
            alias="reeferAmbientAirTemperatureMilliC", description="A list of ambient air temperature points."
        ),
    ] = None
    reefer_door_state_zone_1: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferDoorStateZone1WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferDoorStateZone1"),
        pydantic.Field(alias="reeferDoorStateZone1", description="A list of door state points."),
    ] = None
    reefer_door_state_zone_2: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferDoorStateZone2WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferDoorStateZone2"),
        pydantic.Field(alias="reeferDoorStateZone2", description="A list of door state points."),
    ] = None
    reefer_door_state_zone_3: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferDoorStateZone3WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferDoorStateZone3"),
        pydantic.Field(alias="reeferDoorStateZone3", description="A list of door state points."),
    ] = None
    reefer_fuel_percent: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferFuelPercentWithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferFuelPercent"),
        pydantic.Field(alias="reeferFuelPercent", description="A list of fuel percent points."),
    ] = None
    reefer_obd_engine_seconds: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferObdEngineSecondsWithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferObdEngineSeconds"),
        pydantic.Field(alias="reeferObdEngineSeconds", description="A list of engine second points"),
    ] = None
    reefer_return_air_temperature_milli_c_zone_1: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferReturnAirTemperatureMilliCZone1WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferReturnAirTemperatureMilliCZone1"),
        pydantic.Field(
            alias="reeferReturnAirTemperatureMilliCZone1", description="A list of return air temperature points."
        ),
    ] = None
    reefer_return_air_temperature_milli_c_zone_2: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferReturnAirTemperatureMilliCZone2WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferReturnAirTemperatureMilliCZone2"),
        pydantic.Field(
            alias="reeferReturnAirTemperatureMilliCZone2", description="A list of return air temperature points."
        ),
    ] = None
    reefer_return_air_temperature_milli_c_zone_3: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferReturnAirTemperatureMilliCZone3WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferReturnAirTemperatureMilliCZone3"),
        pydantic.Field(
            alias="reeferReturnAirTemperatureMilliCZone3", description="A list of return air temperature points."
        ),
    ] = None
    reefer_run_mode: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferRunModeWithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferRunMode"),
        pydantic.Field(alias="reeferRunMode", description="A list of run mode points"),
    ] = None
    reefer_set_point_temperature_milli_c_zone_1: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferSetPointTemperatureMilliCZone1WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferSetPointTemperatureMilliCZone1"),
        pydantic.Field(
            alias="reeferSetPointTemperatureMilliCZone1", description="A list of set point temperature points."
        ),
    ] = None
    reefer_set_point_temperature_milli_c_zone_2: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferSetPointTemperatureMilliCZone2WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferSetPointTemperatureMilliCZone2"),
        pydantic.Field(
            alias="reeferSetPointTemperatureMilliCZone2", description="A list of set point temperature points."
        ),
    ] = None
    reefer_set_point_temperature_milli_c_zone_3: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferSetPointTemperatureMilliCZone3WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferSetPointTemperatureMilliCZone3"),
        pydantic.Field(
            alias="reeferSetPointTemperatureMilliCZone3", description="A list of set point temperature points."
        ),
    ] = None
    reefer_state_zone_1: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferStateZone1WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferStateZone1"),
        pydantic.Field(alias="reeferStateZone1", description="A list of engine state points."),
    ] = None
    reefer_state_zone_2: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferStateZone2WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferStateZone2"),
        pydantic.Field(alias="reeferStateZone2", description="A list of engine state points."),
    ] = None
    reefer_state_zone_3: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferStateZone3WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferStateZone3"),
        pydantic.Field(alias="reeferStateZone3", description="A list of engine state points."),
    ] = None
    reefer_supply_air_temperature_milli_c_zone_1: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferSupplyAirTemperatureMilliCZone1WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferSupplyAirTemperatureMilliCZone1"),
        pydantic.Field(
            alias="reeferSupplyAirTemperatureMilliCZone1", description="A list of supply air temperature points."
        ),
    ] = None
    reefer_supply_air_temperature_milli_c_zone_2: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferSupplyAirTemperatureMilliCZone2WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferSupplyAirTemperatureMilliCZone2"),
        pydantic.Field(
            alias="reeferSupplyAirTemperatureMilliCZone2", description="A list of supply air temperature points."
        ),
    ] = None
    reefer_supply_air_temperature_milli_c_zone_3: typing_extensions.Annotated[
        typing.Optional[typing.List[TrailerStatReeferSupplyAirTemperatureMilliCZone3WithDecorationsTypeResponseBody]],
        FieldMetadata(alias="reeferSupplyAirTemperatureMilliCZone3"),
        pydantic.Field(
            alias="reeferSupplyAirTemperatureMilliCZone3", description="A list of supply air temperature points."
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .goa_attribute_tiny_response_body import GoaAttributeTinyResponseBody
from .goa_tag_tiny_response_response_body import GoaTagTinyResponseResponseBody


class DetectionLogAssetObjectResponseBody(UniversalBaseModel):
    """
    Asset that the detection is tied to. Always returned.
    """

    attributes: typing.Optional[typing.List[GoaAttributeTinyResponseBody]] = pydantic.Field(default=None)
    """
    List of attributes associated with the entity
    """

    id: str = pydantic.Field()
    """
    Unique ID for the asset object that is reporting the detection. Always returned.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name for the asset object that is reporting the safety event. Only returned when includeAsset is set to true
    """

    tags: typing.Optional[typing.List[GoaTagTinyResponseResponseBody]] = pydantic.Field(default=None)
    """
    Tags for the asset associated with the safety event. Only returned when includeAsset is set to true
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .location_response_response_body import LocationResponseResponseBody
from .trip_asset_response_body import TripAssetResponseBody
from .trip_response_body_completion_status import TripResponseBodyCompletionStatus


class TripResponseBody(UniversalBaseModel):
    """
    Trip
    """

    asset: TripAssetResponseBody
    completion_status: typing_extensions.Annotated[
        TripResponseBodyCompletionStatus,
        FieldMetadata(alias="completionStatus"),
        pydantic.Field(
            alias="completionStatus", description="Trip completion status  Valid values: `inProgress`, `completed`"
        ),
    ]
    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="[RFC 3339] Time the trip was created in Samsara in UTC."),
    ]
    end_location: typing_extensions.Annotated[
        typing.Optional[LocationResponseResponseBody],
        FieldMetadata(alias="endLocation"),
        pydantic.Field(alias="endLocation"),
    ] = None
    start_location: typing_extensions.Annotated[
        LocationResponseResponseBody, FieldMetadata(alias="startLocation"), pydantic.Field(alias="startLocation")
    ]
    trip_end_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="tripEndTime"),
        pydantic.Field(alias="tripEndTime", description="[RFC 3339] Time the trip ended in UTC."),
    ] = None
    trip_start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="tripStartTime"),
        pydantic.Field(alias="tripStartTime", description="[RFC 3339] Time the trip started in UTC."),
    ]
    updated_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime",
            description="[RFC 3339] Time the trip was updated in Samsara in UTC. Valid updates are when `endTime` populates or `completionStatus` changes values.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .safety_event_behavior_labels_response_body_type import SafetyEventBehaviorLabelsResponseBodyType


class SafetyEventBehaviorLabelsResponseBody(UniversalBaseModel):
    """
    Behavior label for a safety event.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the behavior label.
    """

    type: typing.Optional[SafetyEventBehaviorLabelsResponseBodyType] = pydantic.Field(default=None)
    """
    Type of the BehaviorLabel.  Valid values: `Acceleration`, `Braking`, `Crash`, `DefensiveDriving`, `DidNotYield`, `Drinking`, `Drowsy`, `Eating`, `EatingDrinking`, `EdgeDistractedDriving`, `EdgeRailroadCrossingViolation`, `FollowingDistance`, `FollowingDistanceModerate`, `FollowingDistanceSevere`, `ForwardCollisionWarning`, `GenericDistraction`, `GenericTailgating`, `HarshTurn`, `HighSpeedSuddenDisconnect`, `Invalid`, `LaneDeparture`, `LateResponse`, `MobileUsage`, `NearCollison`, `NoSeatbelt`, `ObstructedCamera`, `OperationalEvent`, `Passenger`, `PolicyViolationMask`, `ProtectiveEquipment`, `RanRedLight`, `RearCollisionWarning`, `RollingStop`, `RolloverProtection`, `Smoking`, `Speeding`, `VehicleInBlindSpotWarning`, `VulnerableRoadUserCollisionWarning`, `YawControl`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

V1VisionStepResultsItemCaliperFilterPolarity = typing.Union[
    typing.Literal["LIGHT TO DARK", "DARK TO LIGHT"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .service_task_instance_object_response_body import ServiceTaskInstanceObjectResponseBody
from .work_order_attachment_object_response_body import WorkOrderAttachmentObjectResponseBody
from .work_order_discount_object_response_body import WorkOrderDiscountObjectResponseBody
from .work_order_item_object_response_body import WorkOrderItemObjectResponseBody
from .work_order_object_response_body_priority import WorkOrderObjectResponseBodyPriority
from .work_order_object_response_body_status import WorkOrderObjectResponseBodyStatus
from .work_order_tax_object_response_body import WorkOrderTaxObjectResponseBody
from .work_order_unallocated_labor_object_response_body import WorkOrderUnallocatedLaborObjectResponseBody


class WorkOrderObjectResponseBody(UniversalBaseModel):
    """
    A work order object.
    """

    archived_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="archivedAtTime"),
        pydantic.Field(alias="archivedAtTime", description="The time the work order was archived in RFC 3339 format."),
    ] = None
    asset_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="assetId"), pydantic.Field(alias="assetId", description="The ID of the asset.")
    ]
    assigned_user_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assignedUserId"),
        pydantic.Field(alias="assignedUserId", description="The ID of the assigned mechanic."),
    ] = None
    attachments: typing.Optional[typing.List[WorkOrderAttachmentObjectResponseBody]] = pydantic.Field(default=None)
    """
    Files attached to the work order.
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The category of the work order
    """

    closing_notes: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="closingNotes"),
        pydantic.Field(alias="closingNotes", description="Notes on the work order."),
    ] = None
    completed_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="completedAtTime"),
        pydantic.Field(
            alias="completedAtTime", description="The time the work order was completed in RFC 3339 format."
        ),
    ] = None
    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="The time the work order was created in RFC 3339 format."),
    ]
    created_by_user_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="createdByUserId"),
        pydantic.Field(alias="createdByUserId", description="The ID of the creator of the work order."),
    ] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A description of what needs to be fixed.
    """

    discount: typing.Optional[WorkOrderDiscountObjectResponseBody] = None
    due_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="dueAtTime"),
        pydantic.Field(alias="dueAtTime", description="The due date of the work order in RFC 3339 format."),
    ] = None
    engine_hours: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="engineHours"),
        pydantic.Field(alias="engineHours", description="The engine hours at the time of the work order."),
    ] = None
    id: str = pydantic.Field()
    """
    The unique id of the work order.
    """

    invoice_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="invoiceNumber"),
        pydantic.Field(alias="invoiceNumber", description="The invoice number for the work order."),
    ] = None
    items: typing.Optional[typing.List[WorkOrderItemObjectResponseBody]] = pydantic.Field(default=None)
    """
    Items related to the work order.
    """

    odometer_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="odometerMeters"),
        pydantic.Field(alias="odometerMeters", description="The odometer reading at the time of the work order."),
    ] = None
    po_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="poNumber"),
        pydantic.Field(alias="poNumber", description="The purchase order number for the work order."),
    ] = None
    priority: typing.Optional[WorkOrderObjectResponseBodyPriority] = pydantic.Field(default=None)
    """
    The priority of the work order  Valid values: `High`, `Low`, `Medium`, `Urgent`
    """

    service_task_instances: typing_extensions.Annotated[
        typing.Optional[typing.List[ServiceTaskInstanceObjectResponseBody]],
        FieldMetadata(alias="serviceTaskInstances"),
        pydantic.Field(alias="serviceTaskInstances", description="Service Tasks for the work order."),
    ] = None
    status: WorkOrderObjectResponseBodyStatus = pydantic.Field()
    """
    The status of the work order  Valid values: `Assigned`, `Cancelled`, `Closed`, `Completed`, `In Progress`, `On Hold`, `Open`, `Pending Approval`, `Pending Parts`
    """

    tax: typing.Optional[WorkOrderTaxObjectResponseBody] = None
    unallocated_labor: typing_extensions.Annotated[
        typing.Optional[WorkOrderUnallocatedLaborObjectResponseBody],
        FieldMetadata(alias="unallocatedLabor"),
        pydantic.Field(alias="unallocatedLabor"),
    ] = None
    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime", description="The time the work order was last updated in RFC 3339 format."
        ),
    ]
    vendor_uuid: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="vendorUuid"),
        pydantic.Field(alias="vendorUuid", description="The vendor UUID for the work order."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_asset_cable import V1AssetCable
from .v_1_asset_current_location import V1AssetCurrentLocation


class V1AssetCurrentLocationsResponse(UniversalBaseModel):
    """
    Basic information of an asset
    """

    asset_serial_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assetSerialNumber"),
        pydantic.Field(alias="assetSerialNumber", description="Asset serial number"),
    ] = None
    cable: typing.Optional[V1AssetCable] = None
    engine_hours: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="engineHours"),
        pydantic.Field(alias="engineHours", description="Engine hours"),
    ] = None
    id: int = pydantic.Field()
    """
    Asset ID
    """

    location: typing.Optional[typing.List[V1AssetCurrentLocation]] = pydantic.Field(default=None)
    """
    Current location of an asset
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Asset name
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .assets_inputs_asset_response_response_body import AssetsInputsAssetResponseResponseBody
from .assets_inputs_aux_input_response_body import AssetsInputsAuxInputResponseBody
from .assets_inputs_response_response_body_units import AssetsInputsResponseResponseBodyUnits


class AssetsInputsResponseResponseBody(UniversalBaseModel):
    """
    Full assets inputs objects.
    """

    asset: AssetsInputsAssetResponseResponseBody
    aux_input: typing_extensions.Annotated[
        typing.Optional[AssetsInputsAuxInputResponseBody],
        FieldMetadata(alias="auxInput"),
        pydantic.Field(alias="auxInput"),
    ] = None
    happened_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="happenedAtTime"),
        pydantic.Field(alias="happenedAtTime", description="UTC timestamp in RFC 3339 format of the event."),
    ]
    units: AssetsInputsResponseResponseBodyUnits = pydantic.Field()
    """
    Units of the values in the returned data.  Valid values: `boolean`, `millivolts`, `microamps`
    """

    value: str = pydantic.Field()
    """
    Value of the data point.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.document_pdf_generation_response import DocumentPdfGenerationResponse
from ..types.document_pdf_query_response import DocumentPdfQueryResponse
from ..types.document_types_get_document_types_response_body import DocumentTypesGetDocumentTypesResponseBody
from ..types.documents_get_document_response_body import DocumentsGetDocumentResponseBody
from ..types.documents_get_documents_response_body import DocumentsGetDocumentsResponseBody
from ..types.documents_post_document_response_body import DocumentsPostDocumentResponseBody
from ..types.field_object_post_request_body import FieldObjectPostRequestBody
from .raw_client import AsyncRawDocumentsClient, RawDocumentsClient
from .types.documents_post_document_request_body_state import DocumentsPostDocumentRequestBodyState

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DocumentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDocumentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDocumentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDocumentsClient
        """
        return self._raw_client

    def get_document_types(
        self, *, after: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> DocumentTypesGetDocumentTypesResponseBody:
        """
        Returns a list of the organization document types. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentTypesByOrgId).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentTypesGetDocumentTypesResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.documents.get_document_types()
        """
        _response = self._raw_client.get_document_types(after=after, request_options=request_options)
        return _response.data

    def get_documents(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        document_type_id: typing.Optional[str] = None,
        query_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DocumentsGetDocumentsResponseBody:
        """
        Get all documents for the given time range. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentsByOrgId).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        document_type_id : typing.Optional[str]
            ID of the document template type.

        query_by : typing.Optional[str]
            Query by document creation time (`created`) or updated time (`updated`). Defaults to `created`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsGetDocumentsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.documents.get_documents(
            start_time="startTime",
            end_time="endTime",
        )
        """
        _response = self._raw_client.get_documents(
            start_time=start_time,
            end_time=end_time,
            after=after,
            document_type_id=document_type_id,
            query_by=query_by,
            request_options=request_options,
        )
        return _response.data

    def post_document(
        self,
        *,
        document_type_id: str,
        driver_id: str,
        fields: typing.Optional[typing.Sequence[FieldObjectPostRequestBody]] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        route_stop_id: typing.Optional[str] = OMIT,
        state: typing.Optional[DocumentsPostDocumentRequestBodyState] = OMIT,
        vehicle_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DocumentsPostDocumentResponseBody:
        """
        Creates a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/createDriverDocument).

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        document_type_id : str
            ID for the document type.

        driver_id : str
            ID of the driver. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        fields : typing.Optional[typing.Sequence[FieldObjectPostRequestBody]]
            The fields associated with this document.

        name : typing.Optional[str]
            Name of the document.

        notes : typing.Optional[str]
            Notes on the document.

        route_stop_id : typing.Optional[str]
            ID of the route stop. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the route stop.

        state : typing.Optional[DocumentsPostDocumentRequestBodyState]
            The condition of the document created for the driver. Can be either `required` or `submitted`, if no value is specified, `state` defaults to `required`. `required` documents are pre-populated documents for the Driver to fill out in the Driver App.  Valid values: `submitted`, `required`

        vehicle_id : typing.Optional[str]
            ID of the vehicle. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsPostDocumentResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.documents.post_document(
            document_type_id="9814a1fa-f0c6-408b-bf85-51dc3bc71ac7",
            driver_id="45646",
        )
        """
        _response = self._raw_client.post_document(
            document_type_id=document_type_id,
            driver_id=driver_id,
            fields=fields,
            name=name,
            notes=notes,
            route_stop_id=route_stop_id,
            state=state,
            vehicle_id=vehicle_id,
            request_options=request_options,
        )
        return _response.data

    def generate_document_pdf(
        self, *, document_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DocumentPdfGenerationResponse:
        """
        Request creation of a document PDF.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        document_id : str
            ID of the document.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentPdfGenerationResponse
            Newly created PDF generation job.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.documents.generate_document_pdf(
            document_id="6c8c0c01-206a-41a4-9d21-15b9978d04cb",
        )
        """
        _response = self._raw_client.generate_document_pdf(document_id=document_id, request_options=request_options)
        return _response.data

    def get_document_pdf(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DocumentPdfQueryResponse:
        """
        Returns generation job status and download URL for a PDF.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the pdf.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentPdfQueryResponse
            Document PDF job status and download URL.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.documents.get_document_pdf(
            id="id",
        )
        """
        _response = self._raw_client.get_document_pdf(id, request_options=request_options)
        return _response.data

    def get_document(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DocumentsGetDocumentResponseBody:
        """
        Returns a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentByIdAndDriverId).

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsGetDocumentResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.documents.get_document(
            id="id",
        )
        """
        _response = self._raw_client.get_document(id, request_options=request_options)
        return _response.data

    def delete_document(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/deleteDriverDocumentByIdAndDriverId).

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the document to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.documents.delete_document(
            id="id",
        )
        """
        _response = self._raw_client.delete_document(id, request_options=request_options)
        return _response.data


class AsyncDocumentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDocumentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDocumentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDocumentsClient
        """
        return self._raw_client

    async def get_document_types(
        self, *, after: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> DocumentTypesGetDocumentTypesResponseBody:
        """
        Returns a list of the organization document types. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentTypesByOrgId).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentTypesGetDocumentTypesResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.get_document_types()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_document_types(after=after, request_options=request_options)
        return _response.data

    async def get_documents(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        document_type_id: typing.Optional[str] = None,
        query_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DocumentsGetDocumentsResponseBody:
        """
        Get all documents for the given time range. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentsByOrgId).

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        document_type_id : typing.Optional[str]
            ID of the document template type.

        query_by : typing.Optional[str]
            Query by document creation time (`created`) or updated time (`updated`). Defaults to `created`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsGetDocumentsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.get_documents(
                start_time="startTime",
                end_time="endTime",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_documents(
            start_time=start_time,
            end_time=end_time,
            after=after,
            document_type_id=document_type_id,
            query_by=query_by,
            request_options=request_options,
        )
        return _response.data

    async def post_document(
        self,
        *,
        document_type_id: str,
        driver_id: str,
        fields: typing.Optional[typing.Sequence[FieldObjectPostRequestBody]] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        route_stop_id: typing.Optional[str] = OMIT,
        state: typing.Optional[DocumentsPostDocumentRequestBodyState] = OMIT,
        vehicle_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DocumentsPostDocumentResponseBody:
        """
        Creates a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/createDriverDocument).

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        document_type_id : str
            ID for the document type.

        driver_id : str
            ID of the driver. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        fields : typing.Optional[typing.Sequence[FieldObjectPostRequestBody]]
            The fields associated with this document.

        name : typing.Optional[str]
            Name of the document.

        notes : typing.Optional[str]
            Notes on the document.

        route_stop_id : typing.Optional[str]
            ID of the route stop. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the route stop.

        state : typing.Optional[DocumentsPostDocumentRequestBodyState]
            The condition of the document created for the driver. Can be either `required` or `submitted`, if no value is specified, `state` defaults to `required`. `required` documents are pre-populated documents for the Driver to fill out in the Driver App.  Valid values: `submitted`, `required`

        vehicle_id : typing.Optional[str]
            ID of the vehicle. Can be either unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsPostDocumentResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.post_document(
                document_type_id="9814a1fa-f0c6-408b-bf85-51dc3bc71ac7",
                driver_id="45646",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_document(
            document_type_id=document_type_id,
            driver_id=driver_id,
            fields=fields,
            name=name,
            notes=notes,
            route_stop_id=route_stop_id,
            state=state,
            vehicle_id=vehicle_id,
            request_options=request_options,
        )
        return _response.data

    async def generate_document_pdf(
        self, *, document_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DocumentPdfGenerationResponse:
        """
        Request creation of a document PDF.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        document_id : str
            ID of the document.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentPdfGenerationResponse
            Newly created PDF generation job.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.generate_document_pdf(
                document_id="6c8c0c01-206a-41a4-9d21-15b9978d04cb",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.generate_document_pdf(
            document_id=document_id, request_options=request_options
        )
        return _response.data

    async def get_document_pdf(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DocumentPdfQueryResponse:
        """
        Returns generation job status and download URL for a PDF.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the pdf.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentPdfQueryResponse
            Document PDF job status and download URL.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.get_document_pdf(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_document_pdf(id, request_options=request_options)
        return _response.data

    async def get_document(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DocumentsGetDocumentResponseBody:
        """
        Returns a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/getDriverDocumentByIdAndDriverId).

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the document

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DocumentsGetDocumentResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.get_document(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_document(id, request_options=request_options)
        return _response.data

    async def delete_document(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a single document. The legacy version of this endpoint can be found at [samsara.com/api-legacy](https://www.samsara.com/api-legacy#operation/deleteDriverDocumentByIdAndDriverId).

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Documents** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the document to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.documents.delete_document(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_document(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .alert_object_driver_response_body import AlertObjectDriverResponseBody
from .alert_object_trailer_response_body import AlertObjectTrailerResponseBody
from .alert_object_vehicle_response_body import AlertObjectVehicleResponseBody


class ReeferTemperatureResponseBody(UniversalBaseModel):
    """
    Details specific to Reefer Temperature.
    """

    driver: typing.Optional[AlertObjectDriverResponseBody] = None
    trailer: typing.Optional[AlertObjectTrailerResponseBody] = None
    vehicle: typing.Optional[AlertObjectVehicleResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .sensor import Sensor
from .vehicle_sensor_configuration_area_position import VehicleSensorConfigurationAreaPosition


class VehicleSensorConfigurationArea(UniversalBaseModel):
    """
    The sensor configuration for an area of the vehicle
    """

    cargo_sensors: typing_extensions.Annotated[
        typing.Optional[typing.List[Sensor]],
        FieldMetadata(alias="cargoSensors"),
        pydantic.Field(alias="cargoSensors", description="Cargo sensors configured in this position of the vehicle"),
    ] = None
    humidity_sensors: typing_extensions.Annotated[
        typing.Optional[typing.List[Sensor]],
        FieldMetadata(alias="humiditySensors"),
        pydantic.Field(
            alias="humiditySensors", description="Humidity sensors configured in this position of the vehicle"
        ),
    ] = None
    position: typing.Optional[VehicleSensorConfigurationAreaPosition] = pydantic.Field(default=None)
    """
    Position of the area on vehicle
    """

    temperature_sensors: typing_extensions.Annotated[
        typing.Optional[typing.List[Sensor]],
        FieldMetadata(alias="temperatureSensors"),
        pydantic.Field(
            alias="temperatureSensors", description="Temperature sensors configured in this position of the vehicle"
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FormsCheckBoxesValueObjectResponseBody(UniversalBaseModel):
    """
    The value of a check boxes form input field.
    """

    value: typing.List[str] = pydantic.Field()
    """
    List of selected options.
    """

    value_ids: typing_extensions.Annotated[
        typing.List[str],
        FieldMetadata(alias="valueIds"),
        pydantic.Field(alias="valueIds", description="List of selected option IDs."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .defect_photo_response_response_body import DefectPhotoResponseResponseBody
from .defect_trailer_response_response_body import DefectTrailerResponseResponseBody
from .defect_vehicle_response_response_body import DefectVehicleResponseResponseBody
from .dvir_resolved_by_object_response_body import DvirResolvedByObjectResponseBody


class DefectsResponseDataResponseBody(UniversalBaseModel):
    """
    DVIR defects data.
    """

    comment: str = pydantic.Field()
    """
    Comment on the defect.
    """

    created_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="Time when defect was created in RFC 3339 format."),
    ] = None
    defect_photos: typing_extensions.Annotated[
        typing.Optional[typing.List[DefectPhotoResponseResponseBody]],
        FieldMetadata(alias="defectPhotos"),
        pydantic.Field(alias="defectPhotos", description="List of DVIR defect's photos"),
    ] = None
    defect_type_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="defectTypeId"),
        pydantic.Field(alias="defectTypeId", description="The unique ID of the defect type."),
    ] = None
    dvir_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="dvirId"),
        pydantic.Field(alias="dvirId", description="The unique ID of the defect's DVIR."),
    ]
    id: str = pydantic.Field()
    """
    The unique ID of the DVIR defect.
    """

    is_resolved: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isResolved"),
        pydantic.Field(alias="isResolved", description="Signifies if this defect is resolved."),
    ]
    mechanic_notes: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="mechanicNotes"),
        pydantic.Field(alias="mechanicNotes", description="The mechanics notes on the defect."),
    ] = None
    resolved_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="resolvedAtTime"),
        pydantic.Field(
            alias="resolvedAtTime",
            description="Time when this defect was resolved in RFC 3339 format. Will not be returned if the defect is unresolved.",
        ),
    ] = None
    resolved_by: typing_extensions.Annotated[
        typing.Optional[DvirResolvedByObjectResponseBody],
        FieldMetadata(alias="resolvedBy"),
        pydantic.Field(alias="resolvedBy"),
    ] = None
    trailer: typing.Optional[DefectTrailerResponseResponseBody] = None
    updated_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(alias="updatedAtTime", description="Time when defect was last updated in RFC 3339 format."),
    ] = None
    vehicle: typing.Optional[DefectVehicleResponseResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class AddressResponseResponseBody(UniversalBaseModel):
    """
    Closest address that the GPS latitude and longitude match to.
    """

    city: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the city
    """

    country: typing.Optional[str] = pydantic.Field(default=None)
    """
    The country
    """

    neighborhood: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the neighborhood if one exists
    """

    point_of_interest: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="pointOfInterest"),
        pydantic.Field(alias="pointOfInterest", description="Point that may be of interest to the user"),
    ] = None
    postal_code: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="postalCode"),
        pydantic.Field(alias="postalCode", description="The zip code"),
    ] = None
    state: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the state
    """

    street: typing.Optional[str] = pydantic.Field(default=None)
    """
    The street name
    """

    street_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="streetNumber"),
        pydantic.Field(alias="streetNumber", description="Street number of the address"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.create_user_request_roles import CreateUserRequestRoles
from ..types.list_user_roles_response import ListUserRolesResponse
from ..types.list_users_response import ListUsersResponse
from ..types.user_response import UserResponse
from .raw_client import AsyncRawUsersClient, RawUsersClient
from .types.create_user_request_auth_type import CreateUserRequestAuthType
from .types.update_user_request_auth_type import UpdateUserRequestAuthType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersClient
        """
        return self._raw_client

    def list_user_roles(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListUserRolesResponse:
        """
        Returns a list of all user roles in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListUserRolesResponse
            List all user roles.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.users.list_user_roles()
        """
        _response = self._raw_client.list_user_roles(limit=limit, after=after, request_options=request_options)
        return _response.data

    def list_users(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListUsersResponse:
        """
        Returns a list of all users in an organization. Users that have expired access will not be returned.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListUsersResponse
            List all users.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.users.list_users()
        """
        _response = self._raw_client.list_users(limit=limit, after=after, request_options=request_options)
        return _response.data

    def create_user(
        self,
        *,
        auth_type: CreateUserRequestAuthType,
        email: str,
        name: str,
        roles: typing.Sequence[CreateUserRequestRoles],
        expire_at: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserResponse:
        """
        Add a user to the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        auth_type : CreateUserRequestAuthType
            The authentication type the user uses to authenticate. To use SAML this organization must have a configured SAML integration. Valid values: `default`, `saml`.

        email : str
            The email address of this user.

        name : str
            The first and last name of the user.

        roles : typing.Sequence[CreateUserRequestRoles]
            The list of roles that applies to this user. A user may have "organizational" roles, which apply to the user at the organizational level, and "tag-specific" roles, which apply to the user for a given tag.

        expire_at : typing.Optional[str]
            For users with temporary access, this is the expiration datetime in RFC3339 format

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Newly created user object with ID.

        Examples
        --------
        from samsara import CreateUserRequestRoles, Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.users.create_user(
            auth_type="default",
            email="user@company.com",
            name="Bob Smith",
            roles=[
                CreateUserRequestRoles(
                    role_id="8a9371af-82d1-4158-bf91-4ecc8d3a114c",
                )
            ],
        )
        """
        _response = self._raw_client.create_user(
            auth_type=auth_type,
            email=email,
            name=name,
            roles=roles,
            expire_at=expire_at,
            request_options=request_options,
        )
        return _response.data

    def get_user(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> UserResponse:
        """
        Get a specific user's information. Users that have expired access will not be returned.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Returns the specified user.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.users.get_user(
            id="id",
        )
        """
        _response = self._raw_client.get_user(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the given user.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.users.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update_user(
        self,
        id: str,
        *,
        auth_type: typing.Optional[UpdateUserRequestAuthType] = OMIT,
        expire_at: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        roles: typing.Optional[typing.Sequence[CreateUserRequestRoles]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserResponse:
        """
        Update a specific user's information.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the user.

        auth_type : typing.Optional[UpdateUserRequestAuthType]
            The authentication type the user uses to authenticate. To use SAML this organization must have a configured SAML integration. Valid values: `default`, `saml`.

        expire_at : typing.Optional[str]
            For users with temporary access, this is the expiration datetime in RFC3339 format

        name : typing.Optional[str]
            The first and last name of the user.

        roles : typing.Optional[typing.Sequence[CreateUserRequestRoles]]
            The list of roles that applies to this user. A user may have "organizational" roles, which apply to the user at the organizational level, and "tag-specific" roles, which apply to the user for a given tag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Updated user object with ID.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.users.update_user(
            id="id",
        )
        """
        _response = self._raw_client.update_user(
            id, auth_type=auth_type, expire_at=expire_at, name=name, roles=roles, request_options=request_options
        )
        return _response.data


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersClient
        """
        return self._raw_client

    async def list_user_roles(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListUserRolesResponse:
        """
        Returns a list of all user roles in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListUserRolesResponse
            List all user roles.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.list_user_roles()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_user_roles(limit=limit, after=after, request_options=request_options)
        return _response.data

    async def list_users(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListUsersResponse:
        """
        Returns a list of all users in an organization. Users that have expired access will not be returned.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListUsersResponse
            List all users.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.list_users()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_users(limit=limit, after=after, request_options=request_options)
        return _response.data

    async def create_user(
        self,
        *,
        auth_type: CreateUserRequestAuthType,
        email: str,
        name: str,
        roles: typing.Sequence[CreateUserRequestRoles],
        expire_at: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserResponse:
        """
        Add a user to the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        auth_type : CreateUserRequestAuthType
            The authentication type the user uses to authenticate. To use SAML this organization must have a configured SAML integration. Valid values: `default`, `saml`.

        email : str
            The email address of this user.

        name : str
            The first and last name of the user.

        roles : typing.Sequence[CreateUserRequestRoles]
            The list of roles that applies to this user. A user may have "organizational" roles, which apply to the user at the organizational level, and "tag-specific" roles, which apply to the user for a given tag.

        expire_at : typing.Optional[str]
            For users with temporary access, this is the expiration datetime in RFC3339 format

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Newly created user object with ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara, CreateUserRequestRoles

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.create_user(
                auth_type="default",
                email="user@company.com",
                name="Bob Smith",
                roles=[
                    CreateUserRequestRoles(
                        role_id="8a9371af-82d1-4158-bf91-4ecc8d3a114c",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_user(
            auth_type=auth_type,
            email=email,
            name=name,
            roles=roles,
            expire_at=expire_at,
            request_options=request_options,
        )
        return _response.data

    async def get_user(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> UserResponse:
        """
        Get a specific user's information. Users that have expired access will not be returned.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Returns the specified user.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.get_user(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_user(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the given user.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update_user(
        self,
        id: str,
        *,
        auth_type: typing.Optional[UpdateUserRequestAuthType] = OMIT,
        expire_at: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        roles: typing.Optional[typing.Sequence[CreateUserRequestRoles]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserResponse:
        """
        Update a specific user's information.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Users** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Unique identifier for the user.

        auth_type : typing.Optional[UpdateUserRequestAuthType]
            The authentication type the user uses to authenticate. To use SAML this organization must have a configured SAML integration. Valid values: `default`, `saml`.

        expire_at : typing.Optional[str]
            For users with temporary access, this is the expiration datetime in RFC3339 format

        name : typing.Optional[str]
            The first and last name of the user.

        roles : typing.Optional[typing.Sequence[CreateUserRequestRoles]]
            The list of roles that applies to this user. A user may have "organizational" roles, which apply to the user at the organizational level, and "tag-specific" roles, which apply to the user for a given tag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Updated user object with ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.update_user(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_user(
            id, auth_type=auth_type, expire_at=expire_at, name=name, roles=roles, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .behavior_count_details_request_body_comparison import BehaviorCountDetailsRequestBodyComparison


class BehaviorCountDetailsRequestBody(UniversalBaseModel):
    """
    Trigger when behavior count meets the specified condition.
    """

    comparison: BehaviorCountDetailsRequestBodyComparison = pydantic.Field()
    """
    The comparison to use when comparing the value to the threshold.  Valid values: `EQUAL_TO`, `GREATER_THAN`, `GREATER_THAN_OR_EQUAL_TO`, `LESS_THAN`, `LESS_THAN_OR_EQUAL_TO`
    """

    num_behaviors: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="numBehaviors"),
        pydantic.Field(alias="numBehaviors", description="The number of behaviors to compare to."),
    ]
    num_days: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="numDays"),
        pydantic.Field(alias="numDays", description="The number of days to compare to."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
